@@ -1742,7 +1742,7 @@ sub interlinear
 
   ########## AN EXAMPLE OF SETTINGS TO BE PUT IN A CONFIGURATION FILE FOLLOWS.
   $maxloops= 1000;
-  $sourcefile = "./sourcefile.csv";
+  $sourcefile = "./caravantrials.csv"; # Name of the source file. It is specified at launch, the value here specified is going to be overridden.
   $newfile = $sourcefile . "_meta.csv";
   $report = $newfile . "_report.txt";
   @mode = ( "wei" ); # #"wei" is weighted gradient linear interpolation of the nearest neighbours.
@@ -2189,25 +2189,18 @@ Or to begin with a dialogue question:
 ./Interlinear.pm interstart;
 .
 
-######################
+The minimal operations for utilizing a data series which is incomplete as regards the parameter listings are the following:
 
+1) copy the executable "opt" in the work folder;
 
-How to utilize a data series which is incomplete as regards the parameter listings.
+2) create a configuration file for Sim::OPT by modifying the "caravantrial.pl" file in the "examples" folder of this distribution (it is sufficient to modify the few values signalled by capital letters in the comments) and place it in the work folder;
 
-1) Copy the executable "opt" in the work folder.
+4) copy the .csv file in the work folder;
 
-2) Create a configuration file for Sim::OPT by modifying the "caravantrial.pl" file in the "examples" folder. It is sufficient to modify the values signalled by capital letters in the comments.
+5) launch Sim::OPT in the shell: << ./opt >>;
 
-3) Create a configuration file for Interlinear by modifying the first setting in the file "confinterlinear.pl" in the "examples" folder.
-
-4) Copy the .csv file in the work folder.
-
-5) Launch Sim::OPT in the shell: << ./opt >>
-
-6) When asked, specify the name (with relative path) of the Sim::OPT configuration file. For example:
-./caravantrial.pl
-
-7) Wait.
+6) when asked, specify the name (with relative path) of the Sim::OPT configuration file. For example:
+./filename.pl .
 
 
 =head2 EXPORT
